
import os
import shutil
from .util import *
from .zood import parseConfig,parseMarkdownFiles,caculateFrontNext,getDirTree,urlReplace

def parseDocs(md_dir_name):
    
    directory_tree, markdown_htmls = parseMarkdownFiles(md_dir_name)
    generateDocs(directory_tree,markdown_htmls,md_dir_name)
 
    
def generateDocs(directory_tree,markdown_htmls,md_dir_name):

    config = getZoodConfig()
    html_dir_name = config['html_folder']
    
    index_README_path = os.path.join(md_dir_name,'.','README.md')
    if not os.path.exists(index_README_path):
        printInfo(f'您需要保留 {md_dir_name}/README.md 作为首页')
        exit()

    if os.path.exists(html_dir_name):
        printInfo(f"删除原 {html_dir_name}/")
        shutil.rmtree(html_dir_name)
    os.makedirs(os.path.join(html_dir_name,'articles'))
    os.makedirs(os.path.join(html_dir_name,'js'))
    os.makedirs(os.path.join(html_dir_name,'css'))
    os.makedirs(os.path.join(html_dir_name,'img'))
    
    # 复制图片
    imgs_path = os.path.join(os.path.dirname(__file__),'config','img')
    for root,_,files in os.walk(imgs_path):
        for img in files:
            shutil.copy(os.path.join(root,img),os.path.join(html_dir_name,'img'))
    
    html_template = parseConfig(config,markdown_htmls)
    index_html_path = os.path.join(html_dir_name,'index.html')
    
    flat_paths = [] # 扁平化之后的所有文件的路径
    for item in directory_tree:
        dir_name = list(item.keys())[0]
        files = item[dir_name]
        if dir_name == '.':
            dir_name = md_dir_name
        for file in files:
            flat_paths.append(os.path.join(dir_name,file))
    
    # 目录树
    dir_tree_html = getDirTree(directory_tree,md_dir_name)
    
    front_url, next_url = caculateFrontNext(flat_paths,index_README_path,md_dir_name)
    
    with open(index_html_path,'w',encoding='utf-8') as f:
        # index 的地址做一些修改
        index_html_template = html_template.replace('../../.','')
        next_url = next_url.replace('../..','./articles')
        index_dir_tree_html = dir_tree_html.replace('../..','./articles')
        index_html_template = index_html_template.replace('../..','./articles')
        
        index_html_template = index_html_template.replace('directory-tree-scope',index_dir_tree_html)
        index_html_template = urlReplace(index_html_template,front_url,next_url,'b')
        f.write(index_html_template.replace('html-scope',markdown_htmls[index_README_path]))
    
    html_template = html_template.replace('directory-tree-scope',dir_tree_html)
    for file_path, markdown_html in markdown_htmls.items():
        dir_name = file_path.split(os.sep)[1]
        file_name = file_path.split(os.sep)[2].replace('.md','')
        if dir_name == '.':
            dir_name = md_dir_name
        doc_path = os.path.join(html_dir_name,"articles",dir_name,file_name)
        os.makedirs(doc_path)
        html_path = os.path.join(doc_path,'index.html')
        with open(html_path,'w',encoding='utf-8') as f:
            front_url, next_url = caculateFrontNext(flat_paths,file_path,md_dir_name)
            # print(html_path,front_url,next_url)
            final_html = urlReplace(html_template,front_url,next_url,'ab')
            f.write(final_html.replace('html-scope',markdown_html))
            
    printInfo(f"已生成 {html_dir_name}/, 打开 {html_dir_name}/index.html 查看",color='green')
          
    



import yaml
import os
import re
import json

def readConfigFile(file_path:str):
    if not os.path.exists(file_path):
        printInfo('找不到文件' + file_path)
        exit(0)
    
    with open(file_path, 'r', encoding="utf-8") as f:
        file_data = f.read()

    data = yaml.load(file_data, Loader=yaml.FullLoader)
    return data


def writeConfigFile(data, file_path):
    with open(file_path, 'w', encoding='utf-8') as f:
        yaml.dump(data, f,allow_unicode=True,sort_keys=False)

def sort(yml):
    
    for _,files in yml.items():
        files.sort(key=lambda item: list(item.values())[0])

def printInfo(msg,color='red'):
    if color == 'red':
        print(f'\033[1;31m{msg}\033[0m')
    elif color == 'green':
        print(f'\033[1;32m{msg}\033[0m')
        
def getZoodConfig():
    
    global_config_path = os.path.join(os.path.dirname(__file__),'config','_config.yml')
    
    global_zood_config = readConfigFile(global_config_path)
    md_dir_name = global_zood_config['markdown_folder']
    
    local_config_path = os.path.join(md_dir_name,'_config.yml')
    if os.path.exists(local_config_path):
        config_path = local_config_path
    else:
        config_path = global_config_path
        
    return readConfigFile(config_path)

def caculateFrontNext(flat_paths:list,path,md_dir_name):

    dir_name = path.split(os.sep)[1]
    file_name = path.split(os.sep)[2].replace('.md','')
    if dir_name == '.':
        dir_name = md_dir_name
    path = os.path.join(dir_name,file_name)
    pos = flat_paths.index(path)
    
    front_url = '\".\"'
    next_url = '\".\"'
    
    if pos != 0 :
        front_url = htmlRelativeUrl(flat_paths[pos-1])
    if pos != len(flat_paths)-1:
        next_url = htmlRelativeUrl(flat_paths[pos+1])

    return front_url,next_url
    
def htmlRelativeUrl(url:str):
    new_url = url.replace(os.sep,'/')
    new_url = f'\"../../{new_url}\"'
    return new_url
    
def getDirTree(directory_tree,md_dir_name):
    tree_html = ''
    for item in directory_tree:
        dir_name = list(item.keys())[0]
        files = item[dir_name]
        if dir_name == '.':
            dir_name = md_dir_name
            for file in files:
                dir_url_link = f'../../{dir_name}/{file}'
                # print(dir_url_link)
                tree_html += treeItem(file,dir_url_link)
        else:
            sub_tree_html = ''
            for file in files:
                dir_url_link = f'../../{dir_name}/{file}'
                # print(dir_url_link)
                sub_tree_html += treeItem(file,dir_url_link)

            first_dir_url_link = f'../../{dir_name}/{files[0]}'
            tree_html += treeItem(dir_name,first_dir_url_link,sub_tree = sub_tree_html)
                
    # print(tree_html)
    return f'<div class=\"dir-tree\">{tree_html}</div>'

def treeItem(name,dir_url_link,sub_tree = False):
    if sub_tree:
        link = f"<a href=\"{dir_url_link}\" >{name}</a>"
        return f'<ul><li>{link}{sub_tree}</li></ul>'
    else:
        link = f"<a href=\"{dir_url_link}\" >{name}</a>"
        return f'<ul><li>{link}</li></ul>'

def urlReplace(html_template,front_url,next_url,control):
    
    html_template = html_template.replace('<%front_url%>',front_url).replace('<%next_url%>',next_url)
    html_template = html_template.replace('<%control%>',f'\"{control}\"')
    return html_template

def getAllAPIText(markdown_htmls,search_scope,md_dir_name):
         
    all_keys = markdown_htmls.keys()
    API_text = {}
    for key in all_keys:
        splited_key = key.split(os.sep)
        dir_name = splited_key[1]
        
        if len(search_scope) != 0:
            if dir_name not in search_scope:
                continue
        if dir_name == '.':
            dir_name = md_dir_name
        file_name = splited_key[2].replace('.md','')
        markdown_text = re.sub(r'<.*?>','',markdown_htmls[key]).replace('\"','').replace('\'','').replace('\\','')
        path = f'../../{dir_name}/{file_name}'
        API_text[path] = markdown_text
    
    json_API_text = json.dumps(API_text).replace('\"','\\\"').replace('\\n','')
    return f'\"{json_API_text}\"'